/*
	Game Categories v 12.0
	Copyright (C) 2009, Bubbletune

	game_categories_macros.h: Game Categories Macros
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __GAME_CATEGORIES_MACROS_H__
#define __GAME_CATEGORIES_MACROS_H__
#include <systemctrl.h>

// Opcode generation
#define MAKE_CALL(a, f) _sw(0x0C000000 | (((u32)(f) >> 2)  & 0x03ffffff), a);
#define MAKE_JUMP(a, f) _sw(0x08000000 | (((u32)(f) & 0x0ffffffc) >> 2), a); 
#define REDIRECT_FUNCTION(a, f) { u32 address = a; _sw(0x08000000 | (((u32)(f) >> 2)  & 0x03ffffff), address);  _sw(0, address+4); }

// Opcode extraction
#define U_EXTRACT_IMPORT(x) ((((u32)_lw((u32)x)) & ~0x08000000) << 2)
#define K_EXTRACT_IMPORT(x) (((((u32)_lw((u32)x)) & ~0x08000000) << 2) | 0x80000000)
#define U_EXTRACT_CALL(x) ((((u32)_lw((u32)x)) & ~0x0C000000) << 2)

// Semaphores
#define LOCK_SEMA(x) { SET_K1_SRL16; sceKernelWaitSema(x, 1, NULL); RESET_K1; }
#define UNLOCK_SEMA(x) { SET_K1_SRL16; sceKernelSignalSema(x, 1); RESET_K1; }
#define LOCK_SEMA_INTERNAL(x) sceKernelWaitSema(x, 1, NULL);
#define UNLOCK_SEMA_INTERNAL(x) sceKernelSignalSema(x, 1);

/* Utopia Source Codes */
#define STR(x) #x
#define GPREG_K1 27
#define GPREG_RA 31

#define _GET_GPREG(r) \
	({ \
		int res; \
		__asm__ __volatile__ \
		( \
			".set push"           "\n" \
			".set noreorder"      "\n" \
      ".frame $sp, 0, $31"  "\n" \
      "move %0,$%1"         "\n" \
			".set pop"            "\n" \
			: "=r"(res) \
			: "i"(r) \
			: "memory" \
		); \
		res; \
	})

#define _SET_GPREG(reg, val) \
	({ \
		__asm__ __volatile__ \
		( \
			".set push"           "\n" \
			".set noreorder"      "\n" \
      ".frame $sp, 0, $31"  "\n" \
      "move $" STR(reg) ", %0"          "\n" \
			".set pop"            "\n" \
			: \
			: "r"(val) \
		); \
	})

#define SET_K1_SRL16  u32 old_k1 = _GET_GPREG(GPREG_K1); u32 k1 = old_k1 >> 16; _SET_GPREG(GPREG_K1, k1);
#define RESET_K1 _SET_GPREG(GPREG_K1, old_k1);

#endif

