/*
	Game Categories v 12.0
	Copyright (C) 2009, Bubbletune

	main.c (loader): Loads the rest of the plugin

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <pspsdk.h>
#include <pspkernel.h>
#include <game_categories.h>

#include <string.h>

PSP_MODULE_INFO("MyCustomHidder", 0x1006, 0, 0);

u32 *start_module_addr;
u32 start_module[3];
int (* StartModule)(SceUID modid, SceSize argsize, void *argp, int *status, SceKernelSMOption *option) = (void *)start_module;
int sceKernelGetSystemStatus();

extern int size_user;
extern int size_kernel;
extern unsigned char user[];
extern unsigned char kernel[];

void ClearCaches()
{
	sceKernelDcacheWritebackAll();
	sceKernelIcacheClearAll();
}

int SelfDestruct(SceSize args, void *argp)
{
	while (sceKernelGetSystemStatus() != 0x20000)
	{
		sceKernelDelayThread(1000);
	}

	sceKernelSelfStopUnloadModule(0, 0, NULL);
	return sceKernelExitDeleteThread(0);
}

int LoadModuleBuffer(void *buf, SceSize size)
{
	SceUID modid = sceKernelLoadModuleBuffer((void *)size, (SceSize)buf, 0, NULL);

	if (modid < 0)
	{
		return modid;
	}

	void (* pspKernelIcacheClearAll)(void) = (void *)(((_lw((u32)sceKernelIcacheClearAll) & ~0x08000000) << 2) | 0x80000000);
	return StartModule(modid, sizeof(pspKernelIcacheClearAll), &pspKernelIcacheClearAll, NULL, NULL);
}

int kernel_loaded = 0;

int StartModulePatched(SceUID modid, SceSize argsize, void *argp, int *status, SceKernelSMOption *option)
{
	if (!kernel_loaded)
	{
		LoadModuleBuffer(kernel, size_kernel);
		kernel_loaded = 1;
	}
	else
	{
		if (sceKernelFindModuleByName("scePaf_Module"))
		{
			LoadModuleBuffer(user, size_user);

			start_module_addr[0] = start_module[0];
			start_module_addr[1] = start_module[2];

			ClearCaches();

			SceUID thid = sceKernelCreateThread("SSelfDestruct", SelfDestruct, 0x10, 0x1000, 0, NULL);

			if (thid >= 0)
			{
				sceKernelStartThread(thid, 0, NULL);
			}
		}
	}
	return StartModule(modid, argsize, argp, status, option);
}

int module_start(SceSize args, void *argp)
{
	if (sceKernelInitKeyConfig() == PSP_INIT_KEYCONFIG_VSH)
	{
		start_module_addr = (u32 *)sctrlHENFindFunction("sceModuleManager", "ModuleMgrForKernel", 0x50F0C1EC);

		if (start_module_addr)
		{
			start_module[0] = start_module_addr[0];
			MAKE_JUMP((u32)&start_module[1], &start_module_addr[2]);
			start_module[2] = start_module_addr[1];
			REDIRECT_FUNCTION((u32)start_module_addr, StartModulePatched);
			ClearCaches();
			return 0;
		}
	}
	return 1;
}

